/*---------------------------------------------------------------------------*/
/* FVDDBF.CMD                                                                */
/*                                                                           */
/*            5763-VR1 COPYRIGHT IBM CORP. 1994.                             */
/*            LICENSED MATERIAL - PROGRAM PROPERTY OF IBM                    */
/*                                                                           */
/*                                                                           */
/*  Description:   This cmd file is to turn the trace on or off for VRPG.    */
/*  The program search the build-time and run-time .ini file for the install */
/*  directory of the build-time or run-time. Then it goes ahead and tries to */
/*  do the specified action for build-time, run-time, or both. If no install */
/*  directory is found for either build-time or run-time, an corrupt .ini    */
/*  file error is returned.                                                  */
/*                                                                           */
/*  Input:        /on   to turn the debug mode on                            */
/*                /off  to turn the debug mode off                           */
/*  Return Code:  0     no error                                             */
/*                1     error                                                */
/*                                                                           */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Echo DOS commands on the screen                                           */
/*---------------------------------------------------------------------------*/
'@echo off'

/*---------------------------------------------------------------------------*/
/* Define the filename of the dlls here                                      */
/* this is no longer needed since we search for the dlls in the directory    */
/* directly. Even if some dlls are missing for debug, no warnings will be    */
/* generated.                                                                */
/*---------------------------------------------------------------------------*/
Build_Files='fvdeca fvdecc fvdecg'
RunTime_Files='fvdeca fvdecc fvdecg'

/*---------------------------------------------------------------------------*/
/* Const Strings used in the program                                         */
/*---------------------------------------------------------------------------*/
msg.ERR_BAD_INI_FILE='Error: A VRPG Client/2 .ini file is corrupted.'

/*---------------------------------------------------------------------------*/
/* define the two signal files for run-time and build-time.                  */
/*---------------------------------------------------------------------------*/
SIG_FVDERDBG = 'FVDERDBG.ON'
SIG_FVDEBDBG = 'FVDEBDBG.ON'

/*---------------------------------------------------------------------------*/
/* Define the system functions                                               */
/* Define the fvderexx utility functions to                                  */
/*---------------------------------------------------------------------------*/
call rxfuncadd sysloadfuncs, rexxutil, sysloadfuncs
call rxfuncadd fvdeloadfuncs, fvderexx, fvdeloadfuncs
call sysloadfuncs
call fvdeloadfuncs

/*---------------------------------------------------------------------------*/
/* Get the information about how this program was called                     */
/*---------------------------------------------------------------------------*/
Parse Upper Source environ . srcdrive':'srcpn'.'srcext
srcpath = Substr(srcpn,1,Lastpos('\',srcpn)-1)
srcname = Substr(srcpn,Lastpos('\',srcpn)+1)

/*---------------------------------------------------------------------------*/
/* Parse parameters                                                          */
/*---------------------------------------------------------------------------*/
Parse Upper Arg Parms

/*---------------------------------------------------------------------------*/
/* get the boot drive of the system                                          */
/* the output is in the form d                                               */
/* this call is available through fvderexx lib.                              */
/*---------------------------------------------------------------------------*/
C_Boot_Drive = fvdegetbootdrive()
Boot_Drive = C_Boot_Drive || ':'

/*---------------------------------------------------------------------------*/
/* initialize the debug on and off flag                                      */
/*---------------------------------------------------------------------------*/
B_Debug_On = 0
B_Debug_Off = 0

/*---------------------------------------------------------------------------*/
/* get number of parameters                                                  */
/*---------------------------------------------------------------------------*/
NumOfParms = words(Parms)

/*---------------------------------------------------------------------------*/
/* Display help if no arguments                                              */
/*---------------------------------------------------------------------------*/
If NumOfParms = 0 Then Do
   signal Help_Message
   exit 0
End /* end do */
Else Do
   If NumOfParms = 1 Then Do
      /* get the first parameter */
      Parm = word( Parms, 1);
      If Parm = '/ON' Then Do
         B_Debug_On = 1
      End /* end do */
      Else Do
         If Parm = '/OFF' Then Do
            B_Debug_Off = 1
         End /* end do */
         Else Do
            /* wrong flag */
            Signal Help_Message
            exit 1
         End /* end do */
      End /* end do */
   End /* end if do */
   Else Do
      /* more than one parameter */
      signal Help_Message
      exit 1
   End /* end else do */
End /* end else do */

/*---------------------------------------------------------------------------*/
/* get the last crash flag.                                                  */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* init variables                                                            */
/* Build_Time = 1 means to turn the debug on or off for build time           */
/* Run_Time = 1 means to turn the debug on or off for the runtime            */
/* B_Ini_File = 0 means that cannot read both ini files for rt and bt.       */
/* B_Error_Found = 0 no error is found                                       */
/*---------------------------------------------------------------------------*/
Build_Time = 1
Run_Time = 1
B_Ini_File = 0
B_Error_Found = 0
/* call the deamon off program */
call Func_Deamon_Off RunTime_Root_Dir

/*---------------------------------------------------------------------------*/
/* get the install directory from install2 ini file for build time           */
/*---------------------------------------------------------------------------*/
Install2_Ini_File = Boot_Drive || "\os2\system\epfis.ini"
ApplicationName='EPFINST_ADTS CS/400_5763-CL1_0600'
KeyName='FilePath'

result = SysIni( Install2_Ini_File, ApplicationName, KeyName )
If ( result = 'ERROR:' | result = '' ) Then Do
   /* Error_Read_Install2_Ini_File */
   Build_Time = 0
End /* end do */
Else Do
   O_Build_Root_Dir = result
   B_Ini_File = 1
   /* get rid of the funny character at the end */
   call Func_Remove_Funny_Char O_Build_Root_Dir
   Build_Root_Dir = result
End /* end do */

/*---------------------------------------------------------------------------*/
/* turn the debug on or off flag for the build time                          */
/*---------------------------------------------------------------------------*/
if Build_Time = 1 Then
Do
   If B_Debug_On = 1 Then Do
      call Func_Last_Crash Build_Root_Dir
      if result = 0 then do
         call Func_Debug_On flag.crash Build_Root_Dir SIG_FVDEBDBG Build_Files
      End
   End
   If B_Debug_Off = 1 Then Do
      call Func_Last_Crash Build_Root_Dir
      if result = 0 then do
         call Func_Debug_Off flag.crash  Build_Root_Dir SIG_FVDEBDBG Build_Files
      end
   End
   if result <> 0 Then Do
      B_Error_Found = 1
   End /* end do */
   call Func_Erase_Crash Build_Root_Dir
End /* end if do */

/*---------------------------------------------------------------------------*/
/* get the root directory from our ini file for run-time                     */
/*---------------------------------------------------------------------------*/
RT_Ini_File = Boot_Drive || "\os2\system\fvdinst.ini"
ApplicationName='VRPGRT'
KeyName='PATH'

result = SysIni( RT_Ini_File, ApplicationName, KeyName )
If ( result = 'ERROR:' | result = '' ) Then Do
   /* say ' Error_Read_Run_Time_Ini_File ' result */
   Run_Time = 0
End /* end do */
Else Do
   O_RunTime_Root_Dir = result
   B_Ini_File = 1
   /* get rid of the funny character at the end */
   call Func_Remove_Funny_Char O_RunTime_Root_Dir
   RunTime_Root_Dir = result

   /* Runtime dlls are a subset of the build time dlls */
   /* if the run-time is installed in the same directory as the */
   /* build time, we do not need to turn the debug on again for */
   /* the run-time. */
   call Func_UpperCase RunTime_Root_Dir
   RunTime_Root_Dir = result
   call Func_UpperCase Build_Root_Dir
   Build_Root_Dir = result
   CompRc = Compare( RunTime_Root_Dir, Build_Root_Dir )
   if CompRc = 0 Then Do
      Run_Time = 0
   End /* end do */
End /* end do */

if Run_Time = 1 Then
Do
   If B_Debug_On = 1 Then Do
      call Func_Last_Crash RunTime_Root_Dir
      if result = 0 then do
         call Func_Debug_On flag.crash RunTime_Root_Dir SIG_FVDERDBG RunTime_Files
      end
   End /* end do */
   If B_Debug_Off = 1 Then Do
      call Func_Last_Crash RunTime_Root_Dir
      if result = 0 then do
         /* say '<<< test >>> calling runtime debug off'  */
         call Func_Debug_Off flag.crash  RunTime_Root_Dir SIG_FVDERDBG RunTime_Files
      end
   End /* end do */
   if result <> 0 Then Do
      B_Error_Found = 1
   End /* end do */
   If B_Debug_Off = 1 Then Do
      call Func_Deamon_On RunTime_Root_Dir flag.deamon
   End
   call Func_Erase_Crash RunTime_Root_Dir
   /* say '<<< test >>> end calling runtime debug off' result */
End /* end if do */

/*---------------------------------------------------------------------------*/
/* both ini files are bad.                                                   */
/*---------------------------------------------------------------------------*/
If  B_Ini_File = 0 Then Do
   say msg.ERR_BAD_INI_FILE
   B_Error_Found = 1
End /* end do */

call Func_Erase_Crash Build_Root_Dir
/* say 'error_found ' B_Error_Found */
If B_Error_Found = 1 Then
   /* say '<<< test >>> exit 1' */
   exit 1
Else
   /* say '<<< test >>> exit 0' */
   exit 0

/*---------------------------------------------------------------------------*/
/* This function will turn on the debug flag for either build-time or        */
/* run-time.                                                                 */
/* Parameters: 1. Root_Dir, root dir for build-time or run-time              */
/*             2. Signal,   name of the signal file                          */
/*             3. Dll file list                                              */
/*---------------------------------------------------------------------------*/
Func_Debug_On: procedure

   /* say '<<< Test  Entering function debug on >>>' */
   arg flag.crash  Root_Dir Signal Files

   /*------------------------------------------------------------------------*/
   /* Const Strings used in the program                                      */
   /*------------------------------------------------------------------------*/
   call Func_Set_Const_String

   /*------------------------------------------------------------------------*/
   /* set the environment variable                                           */
   /*------------------------------------------------------------------------*/
   'SET ICLUI TRACETO=STDERR'

   /*------------------------------------------------------------------------*/
   /* create a log file to hold the output from pack and unpack              */
   /* this file resides in the root directory of build-time or run-time      */
   /* this file is erased in the beginning of the routine.                   */
   /*------------------------------------------------------------------------*/
   TmpFile = Root_Dir || '\FVDDBF.LOG'
   /*------------------------------------------------------------------------*/
   /* remove the tmp file                                                    */
   /*------------------------------------------------------------------------*/
   'ERASE ' TmpFile '1>NUL 2>&1'

   /*------------------------------------------------------------------------*/
   /* check if the signal file exists                                        */
   /* The singal file resides in the root dir of build-time or run-time      */
   /*------------------------------------------------------------------------*/
   SignalFileName = Root_Dir || '\' || Signal

   ProgRc = SysFileTree(SignalFileName , 'File', 'F')
   If ProgRc = 0 THEN DO
      if File.0 <> 0 Then Do
         /* call Func_Display_Message Signal TmpFile msg.DEBUGON_ALREADY */
         /* return 0 */
      end /* end do */
   End /* end do */
   Else Do
      say msg.ERR_SYSFILETREE
      say msg.ERR_RETURNCODE ProgRc || '.'
      return 1
   End /* end do */

   call Func_Debug_On_0 flag.crash Root_Dir TmpFile Signal 'DLL'
   If result <> 0 Then Do
      return 1
   End /* end else do */
   call Func_Debug_On_0 flag.crash Root_Dir TmpFile Signal 'CTRL'
   If result <> 0 Then Do
      return 1
   End /* end else do */
   call Func_Debug_On_0 flag.crash Root_Dir TmpFile Signal 'BIN'
   If result <> 0 Then Do
      return 1
   End /* end else do */

   /*------------------------------------------------------------------------*/
   /* create the file fvedbg.on                                              */
   /*------------------------------------------------------------------------*/
   'ECHO' msg.DEBUGON_ALREADY '> ' SignalFileName

   return 0

/*---------------------------------------------------------------------------*/
/* This function will turn off the debug flag for either build-time or       */
/* run-time.                                                                 */
/* Parameters: 1. Root_Dir, root dir for build-time or run-time              */
/*             2. Signal,   name of the signal file                          */
/*             3. Dll file list                                              */
/*---------------------------------------------------------------------------*/
Func_Debug_Off: procedure

   /* say '<<< Test  Entering function debug off >>>'  */
   arg flag.crash Root_Dir Signal Files
   /* say '<<< Test  Root dir is ' Root_dir ' >>>' */

   /*------------------------------------------------------------------------*/
   /* Const Strings used in the program                                      */
   /*------------------------------------------------------------------------*/
   call Func_Set_Const_String

   /*------------------------------------------------------------------------*/
   /* unset the environment variable                                         */
   /*------------------------------------------------------------------------*/
   'SET ICLUI TRACETO='

   /*------------------------------------------------------------------------*/
   /* create a tmp file to hold the output from pack and unpack              */
   /*------------------------------------------------------------------------*/
   TmpFile = Root_Dir || '\FVDDBF.LOG'

   /*------------------------------------------------------------------------*/
   /* remove the log file                                                    */
   /*------------------------------------------------------------------------*/
   'ERASE ' TmpFile '1>NUL 2>&1'

   /*------------------------------------------------------------------------*/
   /* check if the signal file exists                                        */
   /* The singal file resides in the root dir of build-time or run-time      */
   /*------------------------------------------------------------------------*/
   SignalFileName = Root_Dir || '\' || Signal

   ProgRc = SysFileTree(SignalFileName , 'File', 'F')
   If ProgRc = 0 THEN DO
      if File.0 = 0 Then Do
         /* call Func_Display_Message Signal TmpFile msg.DEBUGOFF_ALREADY */
         /* return 0 */
      end /* end do */
   end /* end do */
   Else Do
      say msg.ERR_SYSFILETREE
      say msg.ERR_RETURNCODE ProgRc || '.'
      return 1
   End /* end do */

   WarningFlag = 0
   call Func_Debug_Off_0 flag.crash Root_Dir TmpFile Signal 'DLL'
   /* call Func_Set_Flags result */
   If result <> 0 Then Do
      return 1
   End /* end else do */
   call Func_Debug_Off_0 flag.crash Root_Dir TmpFile Signal 'CTRL'
   If result <> 0 Then Do
      return 1
   End /* end else do */
   call Func_Debug_Off_0 flag.crash Root_Dir TmpFile Signal 'BIN'
   If result <> 0 Then Do
      return 1
   End /* end else do */

   /*------------------------------------------------------------------------*/
   /* remove the file fvedbg.on                                              */
   /*------------------------------------------------------------------------*/
   'ERASE ' SignalFileName '1>NUL 2>&1'

   return 0

/*---------------------------------------------------------------------------*/
/* This function is called by debug_on function to do all the work.          */
/*---------------------------------------------------------------------------*/
Func_Debug_On_0: procedure
   parse upper arg flag.crash Root_Dir TmpFile Signal SearchP

   /*------------------------------------------------------------------------*/
   /* Const Strings used in the program                                      */
   /*------------------------------------------------------------------------*/
   call Func_Set_Const_String
   flag.DLL = 0
   flag.DAL = 0
   flag.DLA = 0

   /*------------------------------------------------------------------------*/
   /* get all the .d@L files                                                 */
   /* These files are the ones with debug information.                       */
   /* They replace the corresponding files without debug.                    */
   /* If these files are not present, there is no need to pack the existing  */
   /* dlls.                                                                  */
   /*------------------------------------------------------------------------*/
   if ( ( translate( SearchP ) = 'DLL' ) | ( translate( SearchP ) = 'CTRL' ) ) Then Do
      call Func_Files_Exist Root_Dir SearchP "D@L" "D!L"
      flag.dal = flag.all
      if flag.dal = 0 then do
         /* skip, already done */
         onmsg = msg.DEBUGON_ALREADY Root_Dir || '\' || SearchP || '.'
         call Func_Display_Message Signal TmpFile onmsg
         return 0
      end
      else do
         call Func_Dll_Exist flag.crash Root_Dir SearchP '.D@L' '.DLL' '.DL@' TmpFile Signal
         call Func_Dll_Exist flag.crash Root_Dir SearchP '.D!L' '.DLL' '.DL!' TmpFile Signal
         return result
      end
   End
   if translate( SearchP) = 'BIN' Then Do
      call Func_Files_Exist Root_Dir SearchP "E@E" "E!E" 
      flag.dal = flag.all
      if flag.dal = 0 then do
         /* skip, already done */
         onmsg = msg.DEBUGON_ALREADY Root_Dir || '\' || SearchP || '.'
         call Func_Display_Message Signal TmpFile onmsg
         return 0
      end
      else do
         call Func_Dll_Exist flag.crash Root_Dir SearchP '.E@E' '.EXE' '.EX@' TmpFile Signal
         call Func_Dll_Exist flag.crash Root_Dir SearchP '.E!E' '.EXE' '.EX!' TmpFile Signal
      end
   End
   return 0


/*---------------------------------------------------------------------------*/
/* This function is called by debug_off function to do all the work.         */
/*---------------------------------------------------------------------------*/
Func_Debug_Off_0: procedure
   parse upper arg flag.crash Root_Dir TmpFile Signal SearchP

   /*------------------------------------------------------------------------*/
   /* Const Strings used in the program                                      */
   /*------------------------------------------------------------------------*/
   call Func_Set_Const_String
   flag.DLL = 0
   flag.DAL = 0
   flag.DLA = 0

   /*------------------------------------------------------------------------*/
   /* get all the .dL@ files                                                 */
   /* These files are the ones with nodebug information.                     */
   /* They replace the corresponding files with    debug.                    */
   /* If these files are not present, there is no need to pack the existing  */
   /* dlls.                                                                  */
   /*------------------------------------------------------------------------*/
   if ( ( translate( SearchP ) = 'DLL' ) | ( translate( SearchP ) = 'CTRL' ) ) Then Do
      call Func_Files_Exist Root_Dir SearchP "DL@" "DL!"
      flag.dla = flag.all
      if flag.dla = 0 then do
         /* skip, already done */
         onmsg = msg.DEBUGOFF_ALREADY Root_Dir || '\' || SearchP || '.'
         call Func_Display_Message Signal TmpFile onmsg
         return 0
      end
      else do
         /* say '<<< test >>> call func_Dll_exist_off ' */
         call Func_Dll_Exist_Off flag.crash Root_Dir SearchP '.D@L' '.DLL' '.DL@' TmpFile Signal
         call Func_Dll_Exist_Off flag.crash Root_Dir SearchP '.D!L' '.DLL' '.DL!' TmpFile Signal
         return result
      end
   End
   if translate( SearchP) = 'BIN' Then Do
      call Func_Files_Exist Root_Dir SearchP "Ex@" "EX!"
      flag.dla = flag.all
      if flag.dla = 0 then do
         /* skip, already done */
         onmsg = msg.DEBUGOFF_ALREADY Root_Dir || '\' || SearchP || '.'
         call Func_Display_Message Signal TmpFile onmsg
         return 0
      end
      else do
         call Func_Dll_Exist_Off flag.crash Root_Dir SearchP '.E@E' '.EXE' '.EX@' TmpFile Signal
         call Func_Dll_Exist_Off flag.crash Root_Dir SearchP '.E!E' '.EXE' '.EX!' TmpFile Signal
      end
   End
   return 0

Func_Pack_File: procedure
   parse upper arg Root_Dir SourceDir SourceFile TargetFileName TmpFile
   /* say '<<< test >>> entering unpack procedure ' */
   call Func_Set_Const_String
   /*------------------------------------------------------------------------*/
   /* Name of the pack and unpack programs                                   */
   /*------------------------------------------------------------------------*/
   PackExeName = Root_Dir || '\bin\fvdipkfl.exe'

   PackTmp = SourceDir || "\FVDPACK.TMP"
   /* erase the Target if exist */
   target = SourceDir || '\' || TargetFileName
   'erase ' target '1>NUL 2>&1'
   PackExeName SourceFile PackTmp '1>>' TmpFile '2>&1'
   if RC = 0 Then Do
      /* successful packed */
      /* erase the source file, and rename the packed file */
      'ERASE' SourceFile '1>NUL 2>&1'
      If RC = 0 then Do
         'RENAME' PackTmp TargetFileName '1>NUL 2>&1'
         if RC <> 0 then Do
            /* error removing file */
            say msg.ERR_RENAME_FILE TargetFileName || '.'
            say msg.ERR_RETURNCODE || RC || '.'
            'echo' msg.ERR_RENAME_FILE TargetFileName  ' >> ' TmpFile
            return 1
         End
      End
      Else Do
         say msg.ERR_REMOVE_FILE SourceFile || '.'
         say msg.ERR_RETURNCODE || RC || '.'
         'echo' msg.ERR_REMOVE_FILE SourceFile  ' >> ' TmpFile
         /* remove the tmp file */
         'ATTRIB -R ' PackTmp '1>NUL 2>&1'
         'ERASE ' PackTmp  '1>NUL 2>&1'
         if RC <> 0 then Do
            say msg.ERR_RENAME_FILE TargetFileName || '.'
            say msg.ERR_RETURNCODE || RC || '.'
            'echo' msg.ERR_RENAME_FILE TargetFileName  ' >> ' TmpFile
         End
         return 1
      End
   End
   Else Do
      say msg.ERR_PACK_FILE SourceFile || '.'
      say msg.ERR_RETURNCODE || RC || '.'
      'echo' msg.ERR_PACK_FILE SourceFile  ' >> ' TmpFile
      return 1
   End

   return 0

Func_UnPack_File: procedure
   parse upper arg Root_Dir SourceDir SourceFile TargetFileName TmpFile
   /* say '<<< test >>> entering unpack procedure ' */
   call Func_Set_Const_String
   /*------------------------------------------------------------------------*/
   /* Name of the pack and unpack programs                                   */
   /*------------------------------------------------------------------------*/
   UnPackExeName = Root_Dir || '\bin\fvdiunpk.exe'

   PackTmp = SourceDir || "\\FVDPACK.TMP"
   /* say '<<< test >>> ' UnPackExeName SourceFile SourceDir  */
   UnPackExeName SourceFile SourceDir '1>>' TmpFile '2>&1'
   if RC = 0 Then Do
      /* successful packed */
      /* erase the source file */
      'ERASE' SourceFile '1>NUL 2>&1'
      If RC <> 0 then Do
         /* say '<<< test >>> ahh' */
         say msg.ERR_REMOVE_FILE SourceFile || '.'
         say msg.ERR_RETURNCODE || RC || '.'
         'echo' msg.ERR_REMOVE_FILE SourceFile  ' >> ' TmpFile
         return 1
      End
   End
   Else Do
      say msg.ERR_UNPACK_FILE SourceFile || '.'
      say msg.ERR_RETURNCODE || RC || '.'
      'echo' msg.ERR_UNPACK_FILE SourceFile  ' >> ' TmpFile
      return 1
   End
   return 0

Func_Get_Dll_Pattern: procedure expose OldExt NewExt NewPackExt OldPackExt
   OldExt = '.DLL'
   NewExt = '.D@L'
   NewPackExt = '.D@@'
   OldPackExt = '.DL@'
   return 0

Func_Get_Exe_Pattern: procedure expose OldExt NewExt NewPackExt OldPackExt
   OldExt = '.EXE'
   NewExt = '.E@E'
   NewPackExt = '.E@@'
   OldPackExt = '.EX@'
   return 0

Func_Display_Message: procedure
   parse arg Signal TmpFile Message
   call Func_Set_Const_String
   If translate( Signal ) = 'FVDERDBG.ON' Then Do
      say msg.DEBUG_RUNTIME Message
      'echo' msg.DEBUG_RUNTIME Message ' >> ' TmpFile
   End /* end do */
   If translate( Signal ) = 'FVDEBDBG.ON' Then Do
      say msg.DEBUG_BUILDTIME Message
      'echo' msg.DEBUG_BUILDTIME Message ' >> ' TmpFile
   End /* end do */
   return

Func_Set_Const_String: procedure expose msg.
   msg.DEBUGOFF_SKIP1='Warning : A normal use file cannot be found.'
   msg.DEBUGOFF_SKIP2='Warning : A debug-mode file cannot be found.'
   msg.ERR_REMOVE_FILE='Error: An error occurred when removing file '
   msg.ERR_RESTORE_FILE='Error: An error occurred when restoring file '
   msg.ERR_RENAME_FILE='Error: An error occurred when renaming file '
   msg.ERR_PACK_FILE='Error: An error occurred when packing file '
   msg.ERR_UNPACK_FILE='Error: An error occurred when unpacking file '
   msg.ERR_SYSFILETREE='Error: An error occurred calling function SysFileTree.'
   msg.ERR_RETURNCODE='The return code is '
   msg.DEBUGON_ALREADY='Info : Debug mode is already on for'
   msg.DEBUGON_SUCCESS='The debug-mode files have been successfully generated. '
   msg.DEBUGON_WARNING='The debug-mode files have been generated with warnings.'
   msg.ERR_NO_DEBUGON_FILES='debug-mode files have been unpacked in'
   msg.ERR_NO_DEBUGON_FILES_S='debug-mode file has been unpacked in'
   msg.ERR_DEBUGON_FILES='An error occurred when finding the file name for a debug-mode file.'
   msg.DEBUGOFF_SUCCESS='The normal use files have been successfully restored.'
   msg.DEBUGOFF_WARNING='The normal use files have been restored with warnings.'
   msg.DEBUGOFF_ALREADY='Info : Debug mode is already off for'
   msg.ERR_NO_DEBUGOFF_FILES='normal use files have been restored in'
   msg.ERR_NO_DEBUGOFF_FILES_S='normal use file has been restored in'
   msg.ERR_DEBUGOFF_FILES='An error occurred when finding the file name for a normal use file.'
   msg.DEBUG_RUNTIME='[Run-Time]'
   msg.DEBUG_BUILDTIME='[Build-Time]'
   return 0

Func_Set_Flags: procedure expose WarningFlag NoFileFlag
   parse upper arg n
   if n = 2 then Do
      WarningFlag = 1
   End
   else Do
      if n = 3 then Do
         WarningFlag = 1
         NoFileFlag = 1
      End
   End
   return

Func_Remove_Funny_Char: procedure
   parse upper arg Dir
   slen = LENGTH( Dir )
   lastchar = substr( Dir, slen, 1 )
   Dir = strip( Dir, Traling, D2C( 0 ) )
   /* get rid of any trailing blanks. */
   Root_Dir = strip( Dir, Trailing, ' ' )
   return Root_Dir

Func_Files_Exist: procedure expose flag.
   parse upper arg Root_Dir Pat flagname1 flagname2
   call Func_Set_Const_String
   flag.all = 0
   Pattern = Root_Dir || "\\" || Pat || "\\*." || flagname1

   ProgRc = SysFileTree( Pattern , 'File', 'O')
   If ProgRc = 0 THEN DO
      if File.0 <> 0 Then Do
         flag.all = flag.all + 1
      end /* end do */
   End /* end do */
   Else Do
      say msg.ERR_SYSFILETREE
      say msg.ERR_RETURNCODE ProgRc || '.'
      return 1
   End /* end do */

   Pattern = Root_Dir || "\\" || Pat || "\\*." || flagname2

   ProgRc = SysFileTree( Pattern , 'File', 'O')
   If ProgRc = 0 THEN DO
      if File.0 <> 0 Then Do
         flag.all = flag.all + 1
      end /* end do */
   End /* end do */
   Else Do
      say msg.ERR_SYSFILETREE
      say msg.ERR_RETURNCODE ProgRc || '.'
      return 1
   End /* end do */

   return 0

/*---------------------------------------------------------------------------*/
/* This function displays the help                                           */
/* message for the use of the command                                        */
/*---------------------------------------------------------------------------*/
Help_Message:
   say  srcname || ': turn the debug mode on or off for VRPG Client/2.'
   say 'Usage: '
   say  srcname '[/flag]'
   say ' /on   turn the debug mode on.'
   say ' /off  turn the debug mode off.'
   return 0

/*---------------------------------------------------------------------------*/
/* This function change an input string to uppercase                         */
/*---------------------------------------------------------------------------*/
Func_UpperCase: procedure
   parse upper arg n
   return n

Func_Dll_Exist: procedure expose flag.
   parse upper arg flag.crash Root_Dir SearchP Ext NewExt NewExt2 TmpFile Signal
   flag.all = 0
   Pattern = Root_Dir || '\' || SearchP || '\*' || Ext
   call Func_Set_Const_String
   ProgRc = SysFileTree( Pattern , 'File', 'O')
   If ProgRc = 0 THEN DO
      if File.0 = 0 Then Do
         return 0
      end /* end do */
   End /* end do */
   Else Do
      say msg.ERR_SYSFILETREE
      say msg.ERR_RETURNCODE ProgRc || '.'
      return 1
   End /* end do */
   NumOfFiles = file.0

   Do index=1 to file.0 by 1
      /* create the real file name without debug */
      /* FileName=word( Files, index ) */
      SearchFile = File.index
      /* get the file name stub */
      dotpos = POS( '.' , SearchFile )
      if dotpos = 0 Then Do
         say msg.ERR_DEBUGON_FILES
         'echo' msg.ERR_DEBUGON_FILES   ' >> ' TmpFile
         return 1
      End
      Else Do
         /* get the stud file name */
         FileName = substr( SearchFile, 1, dotpos-1 )
         slashpos = LASTPOS( '\', FileName )
         if slashpos = 0 Then Do
            StubName = FileName
         End /* end do */
         Else Do
            StubName = substr( FileName, slashpos+1 )
         End /* end do */
      End /* end do */

      Pattern = FileName || NewExt
      call Func_Exist Pattern
      if result = 0 then Do
         flag.dll = bexist
      End
      Else Do
         return result
      End
      Pattern = FileName || NewExt2
      call Func_Exist Pattern
      if result = 0 then Do
         flag.dla = bexist
      End
      Else Do
         return result
      End
      if flag.dll = 1 then do
         if ( flag.dla = 1 & flag.crash = 1 ) then do
            /* say ' <<< test >>> crash recovery ' */
            /* the dlls are bad */
            Source = FileName || NewExt /* .dll */
            'erase '  Source '1>NUL 2>&1'
            if RC <> 0 then do
               say msg.ERR_REMOVE_FILE Source || '.'
               say msg.ERR_RETURNCODE || RC || '.'
               'echo' msg.ERR_REMOVE_FILE SourceFile  ' >> ' TmpFile
               return 1
            end
            /* unpack d@l to dll */
            PathPat = Root_Dir || '\' || SearchP
            FileNameNPath = FileName || Ext
            TargetFileName = StubName || NewExt
            /* say '<<< test >>> ' Func_UnPack_File Root_Dir PathPat FileNameNPath TargetFileName TmpFile */
            call Func_Unpack_File Root_Dir PathPat FileNameNPath TargetFileName TmpFile
            if result <> 0 then do
               return 1
            end
         end
         else do
            /* pack .dll to .dl@ */
            /* unpack .d@l to .dll */
            PathPat = Root_Dir || '\' || SearchP
            FileNameNPath = FileName || NewExt
            TargetFileName = StubName || NewExt2
            /* say '<<< test >>> func_dll_exist' */
            /* say '<<< test >>> ' Func_Pack_File Root_Dir PathPat FileNameNPath TargetFileName TmpFile */
            call Func_Pack_File Root_Dir PathPat FileNameNPath TargetFileName TmpFile
            if result <> 0 then do
               return 1
            end
            FileNameNExt = FileName || Ext     /* '.D@L' */
            FileNameNPath = FileNameNExt
            TargetFileName = Root_Dir || '\' || SearchP || '\.'    /* '\DLL\.' */
            /* say '<<< test >>> ' Func_UnPack_File Root_Dir PathPat FileNameNPath TargetFileName TmpFile */
            call Func_UnPack_File Root_Dir PathPat FileNameNPath TargetFileName TmpFile
            If result <> 0 Then Do
               FileNameNExt = FileName || NewExt2        /* '.DL@' */
               FileNameNPath = FileNameNExt
               TargetFileName = StubName || NewExt  /* '\DLL\.' */
               /* say '<<< test >>> ' Func_UnPack_File Root_Dir PathPat FileNameNPath TargetFileName TmpFile */
               call Func_UnPack_File Root_Dir PathPat FileNameNPath TargetFileName TmpFile
               If result <> 0 Then Do
                  return 1
               End /* end do */
            end /* end do */
         end
      end
      else do
         /* unpack d@l to dll */
         PathPat = Root_Dir || '\' || SearchP
         FileNameNPath = FileName || Ext
         TargetFileName = StubName || NewExt
         /* say '<<< test >>> ' Func_UnPack_File Root_Dir PathPat FileNameNPath TargetFileName TmpFile */
         call Func_Unpack_File Root_Dir PathPat FileNameNPath TargetFileName TmpFile
         if result <> 0 then do
            return 1
         end
      end
   end /* end do */

   /*------------------------------------------------------------------------*/
   /* success message                                                        */
   /*------------------------------------------------------------------------*/
   If NumOfFiles = 1 then do
      Msg = NumOfFiles msg.ERR_NO_DEBUGON_FILES_S PathPat || '.'
   End
   Else Do
      Msg = NumOfFiles msg.ERR_NO_DEBUGON_FILES PathPat || '.'
   End
   call Func_Display_Message Signal TmpFile Msg
   return 0

Func_Exist: procedure  expose bexist
   parse upper arg Pattern
   /* say ' entering func_exist' */
   call Func_Set_Const_String
   ProgRc = SysFileTree( Pattern , 'File', 'O')
   If ProgRc = 0 THEN DO
      if File.0 = 0 Then Do
         bexist = 0
      end /* end do */
      else do
         bExist = 1
      End
   End /* end do */
   Else Do
      say msg.ERR_SYSFILETREE
      say msg.ERR_RETURNCODE ProgRc || '.'
      say ' exiting func_exist' ProgRc
      return 1
   End /* end do */

   /* say ' exiting func_exist' bExist */
   return 0

Func_Dll_Exist_Off: procedure expose flag.
   /* say '<<< test >>> entering function func_dll_exist_off ' */
   parse upper arg flag.crash Root_Dir SearchP Ext NewExt NewExt2 TmpFile Signal
   call Func_Set_Const_String
   flag.all = 0
   Pattern = Root_Dir || '\' || SearchP || '\*' || NewExt2
   /* say '<<< test >>> ' pattern */
   ProgRc = SysFileTree( Pattern , 'File', 'O')
   If ProgRc = 0 THEN DO
      if File.0 = 0 Then Do
         return 0
      end /* end do */
   End /* end do */
   Else Do
      say msg.ERR_SYSFILETREE
      say msg.ERR_RETURNCODE ProgRc || '.'
      return 1
   End /* end do */
   NumOfFiles = file.0
   Do index=1 to file.0 by 1
      /* create the real file name without debug */
      /* FileName=word( Files, index ) */
      SearchFile = File.index
      /* get the file name stub */
      dotpos = POS( '.' , SearchFile )
      if dotpos = 0 Then Do
         say msg.ERR_DEBUGON_FILES
         'echo' msg.ERR_DEBUGON_FILES   ' >> ' TmpFile
         return 1
      End
      Else Do
         /* get the stud file name */
         FileName = substr( SearchFile, 1, dotpos-1 )
         slashpos = LASTPOS( '\', FileName )
         if slashpos = 0 Then Do
            StubName = FileName
         End /* end do */
         Else Do
            StubName = substr( FileName, slashpos+1 )
         End /* end do */
      End /* end do */

      Pattern = FileName || NewExt
      call Func_Exist Pattern
      if result = 0 then Do
         flag.dll = bexist
      End
      Else Do
         return result
      End

      Pattern = FileName || Ext
      call Func_Exist Pattern
      if result = 0 then Do
         flag.dal = bexist
      End
      Else Do
         return result
      End
      if flag.dll = 1 then do
         if flag.dal = 0 then do
            /* pack .dll to .d@l */
           /* pack .dl@ to .dll */
            PathPat = Root_Dir || '\' || SearchP
            FileNameNExt = FileName || NewExt      /* '.DLL' */
            FileNameNPath= FileNameNExt
            TargetFile = StubName || Ext        /* '.d@l' */
            /* say '<<< test >>> func_dll_exist' */
            /* say '<<< test >>> ' Func_Pack_File Root_Dir PathPat FileNameNPath TargetFileName TmpFile */
            call Func_Pack_File Root_Dir PathPat FileNameNPath TargetFile TmpFile
            If result <> 0 Then Do
               return 1
            End /* end do */

            FileNameNExt=FileName || NewExt2    /* '.DL@' */
            FileNameNPath = FileNameNExt
            TargetFileName = Root_Dir || '\' || SearchP || '\.'      /* '\DLL\.' */
         /* say '<<< test >>> ' Func_UnPack_File Root_Dir PathPat FileNameNPath TargetFileName TmpFile */
            call Func_UnPack_File Root_Dir PathPat FileNameNPath TargetFileName TmpFile
            If result <> 0 Then Do
               FileNameNExt = FileName || Ext     /* '.D@L' */
               FileNameNPath = FileNameNExt
               TargetFileName = StubName || NewExt
         /* say '<<< test >>> ' Func_UnPack_File Root_Dir PathPat FileNameNPath TargetFileName TmpFile */
               call Func_UnPack_File Root_Dir PathPat FileNameNPath TargetFile TmpFile
               If result <> 0 Then Do
                  return 1
               End /* end do */
               return 1
            End /* end do */
         end
         else Do
            if flag.crash = 0 then do
               PathPat = Root_Dir || '\' || SearchP
               FileNameNPath = FileName || NewExt2
               'erase ' FileNameNPath '1>NUL 2>&1'
               if RC <> 0 then do
                  return 1
               End
            end
            else do
               /* say ' <<< test >>> crash recovery ' */
               /* erase the .dll file */
               FileNameNPath = FileName || NewExt
               'erase ' FileNameNPath '1>NUL 2>&1'
               if RC <> 0 then do
                  say msg.ERR_REMOVE_FILE FileNameNPath || '.'
                  say msg.ERR_RETURNCODE || RC || '.'
                  'echo' msg.ERR_REMOVE_FILE SourceFile  ' >> ' TmpFile
                  return 1
               End
               /* unpack dl@ to dll */
               PathPat = Root_Dir || '\' || SearchP
               FileNameNPath = FileName || NewExt2
               TargetFileName = StubName || NewExt
               /* say '<<< test >>> ' Func_UnPack_File Root_Dir PathPat FileNameNPath TargetFileName TmpFile */
               call Func_Unpack_File Root_Dir PathPat FileNameNPath TargetFileName TmpFile
               if result <> 0 then do
                  return 1
               end
            End
         End
      End
      else do
         /* unpack dl@ to dll */
         PathPat = Root_Dir || '\' || SearchP
         FileNameNPath = FileName || NewExt2
         TargetFileName = StubName || NewExt
         /* say '<<< test >>> ' Func_UnPack_File Root_Dir PathPat FileNameNPath TargetFileName TmpFile */
         call Func_Unpack_File Root_Dir PathPat FileNameNPath TargetFileName TmpFile
         if result <> 0 then do
            return 1
         end
      end
   end /* end do */
   /*------------------------------------------------------------------------*/
   /* success message                                                        */
   /*------------------------------------------------------------------------*/
   if NumOfFiles = 1 then do
      Msg = NumOfFiles  msg.ERR_NO_DEBUGOFF_FILES_S PathPat || '.'
   End
   Else Do
      Msg = NumOfFiles  msg.ERR_NO_DEBUGOFF_FILES PathPat || '.'
   End
   call Func_Display_Message Signal TmpFile Msg
   return 0

Func_Last_Crash: procedure expose flag.
   parse upper arg Root_Dir
   FileName = 'FVDDBCR.ON'
   FileNameNPath = Root_Dir || '\' || FileName
   /*---------------------------------------------------------------------------*/
   /* get the last crash flag.                                                  */
   /*---------------------------------------------------------------------------*/
   call Func_Exist FileNameNPath
   if result <> 0 then do
      return 1
   end
   if bexist = 1 then do
      flag.crash = 1
   end
   else do
      flag.crash = 0
   end
   /* set the crash flag */
   'echo crash > ' FileNameNPath
   return 0

Func_Erase_Crash:procedure
   parse upper arg Root_dir
   FileName = 'FVDDBCR.ON'
   FileNameNPath = Root_Dir || '\' || FileName
   'erase ' FileNameNPath '1>nul 2>&1'
   return Rc

Func_Deamon_On:procedure
   parse upper arg Root_Dir flag.deamon
   'FVDERDMU.EXE on 1>nul 2>&1'
   return 0

Func_Deamon_Off:procedure expose flag.deamon
   parse upper arg Root_Dir
   /* say '<<< test >>> entering func_deamon_off ' */
   'FVDERDMU.EXE off 1>nul 2>&1'
   return 0
