/********************************************************************
*                                                                   *
*  EVFEMARK - a prototype of how to create a 'menu' of commands     *
*             you use most often.                                   *
*                                                                   *
*            (C) Copyright IBM Corporation 1989, 1994               *
*                                                                   *
********************************************************************/

/* Build commands                                                  */
/* - build EVFEMARK.RES : rc -r -x -p evfemark.rc                  */
/* - build EVFEMARK.OBJ : icc /c /Q /Ss /Kb+ /W3 /Gd /Gm /Gn /Fd   */
/*                            /Sp1 /Gh /Ge- /Sm /D__32BIT__ /Ti    */
/*                            /O+ evfemark.c                       */
/* - build EVFEMARK.DLL : link386 /NOI /NOL /M /NOD /NOE           */
/*                            evfemark.obj, evfemark.dll,          */
/*                            evfemark.map, evf4mbsi.lib+          */
/*                            dde4mbso.lib+os2386.lib+             */
/*                            lpexapi.lib, evfemark.def            */
/*                        rc evfemark.res evfemark.dll             */

/* OS/2 includes */
#ifndef OS2H
#define OS2H
#endif

/* system include files */
#include <string.h>

/* editor interface files */
#include "editapi.h"
#include "lpexapi.h"

#include "evfemark.rch"

/* static storage */
/* Array of commands */
CHAR   szCmds[][7] = {"BLOCK",                 /* mark line */
                      "BLOCK",                 /* mark word */
                      "BLOCK",                 /* mark rectangle */
                      "BLOCK",                 /* copy block */
                      "BLOCK",                 /* delete block */
                      "BLOCK",                 /* copy ontop */
                      "BLOCK",                 /* move block */
                      "BLOCK",                 /* uppercase */
                      "BLOCK",                 /* lowercase */
                      "DIALOG",                /* got line dialog */
                      "DIALOG",                /* search dialog */
                      "FIND",                  /* repeat search */
                      "MARK",                  /* set mark" */
                      "MARK"};                 /* find mark */

/* array of parameters to the above commands */
CHAR   szParm[][15] = {"MARK ELEMENT",     /* mark line */
                       "MARK CHARACTER",   /* mark word */
                       "MARK RECTANGLE",   /* mark rectangle */
                       "COPY",             /* copy block */
                       "DELETE",           /* delete block */
                       "OVERLAY",          /* copy ontop */
                       "MOVE",             /* move block */
                       "UPPER",            /* uppercase */
                       "LOWER",            /* lowercase */
                       "ELEMENT",          /* got line dialog */
                       "FINDCHANGE",       /* search dialog */
                       "NEXT",             /* repeat search */
                       "SET WQZXQQ",       /* set mark" */
                       "FIND WQZXQQ"};     /* find mark */

HWND  hwndFrame = NULLHANDLE;

/* function prototypes */
VOID _Optlink MarkDlg(HWND     hwndOwner);
MRESULT EXPENTRY MarkDlgProc(HWND   hwndDlg,
                             ULONG  msg,
                             MPARAM mp1,
                             MPARAM mp2);

/*--------------------------------------------------------------------*/
/*                                                                    */
/* Name:        LScompCall                                            */
/*                                                                    */
/* Description: editor interface to EVFEMARK.DLL                      */
/*                                                                    */
/* Parameters: editor context                                         */
/*             type of request - init, action, terminate              */
/*             line in editor, if applicable                          */
/*             parameter string                                       */
/*             pointer to user data (always NULL)                     */
/*                                                                    */
/* Returns: 0                                                         */
/*                                                                    */
/*--------------------------------------------------------------------*/
ULONG  LScompCall(LXCONTEXT hContext,
                  USHORT    usRequestType,
                  LXLINE    hLine,
                  PSZ       pszParm,
                  PVOID    *pUser)
{
   switch (usRequestType) {
      case LX_LSCOMP_INITIATE:  /* do nothing */
            *pUser = NULL;
         break;

      case LX_LSCOMP_PROCESS:
         MarkDlg(EditQueryCurrentView(hContext));
         break;

      case LX_LSCOMP_TERMINATE:
         break;
   } /* end switch */

   return(0L);
} /* LScompCall */

/*--------------------------------------------------------------------*/
/*                                                                    */
/* Name:        MarkDlg                                               */
/*                                                                    */
/* Description: Display the Mark dialog box .                         */
/*                                                                    */
/* Parameters: caller handle                                          */
/*                                                                    */
/* Returns: 0                                                         */
/*                                                                    */
/*--------------------------------------------------------------------*/
VOID _Optlink MarkDlg(HWND     hwndOwner)
{                                           /* MarkDlg                */
   if (hwndFrame) {
      /* ensure dialog is visible */
      WinShowWindow(hwndFrame, TRUE);
   } else {
      HMODULE hResource;
      CHAR    szLoadError[100];

      /* load the dialog resources from myself.   */
      if (DosLoadModule(szLoadError,
                        sizeof(szLoadError),
                        "EVFEMARK",
                        &hResource) != 0) {
         /* error, display LPEX message and return */
         LpexMsg("Error loading EVFEMARK.DLL resources.");
         return;
      } /* end if */

      hwndFrame = WinLoadDlg(HWND_DESKTOP,
                             HWND_DESKTOP,
                             MarkDlgProc,
                             hResource,
                             IDD_MARK_DIALOG,
                             NULL);

      /* ensure dialog has focus */
      WinSetFocus(HWND_DESKTOP,
                  WinWindowFromID(hwndFrame ? hwndFrame :
                                              hwndOwner,
                                  FID_CLIENT));
   } /* endif */

   return;
}                                           /* MarkDlg                */

/*--------------------------------------------------------------------*/
/*                                                                    */
/* Name:        MarkDlgProc                                           */
/*                                                                    */
/* Description: Invoke the Mark dialog window procedure               */
/*                                                                    */
/* Parameters:  standard PM parms.                                    */
/*                                                                    */
/* Returns:     standard PM rc's                                      */
/*                                                                    */
/* See Also:                                                          */
/*                                                                    */
/*--------------------------------------------------------------------*/
MRESULT EXPENTRY MarkDlgProc(HWND   hwndDlg,
                             ULONG  msg,
                             MPARAM mp1,
                             MPARAM mp2)
{                                               /* MarkDlgProc                   */
   MRESULT mr;

   switch (msg) {
      case WM_INITDLG:
         WinShowWindow(hwndDlg, TRUE);
         mr = (MRESULT) FALSE;
         break;

      case WM_CLOSE:
         (VOID) WinDismissDlg(hwndDlg, SHORT1FROMMP(mp1));
         hwndFrame = NULLHANDLE;
         mr = (MRESULT) NULL;
         break;

      /*--------------------------------------------------------------*/
      /*                                                              */
      /* Process WM_COMMAND message (one of the push buttons was      */
      /* pressed).                                                    */
      /*                                                              */
      /*--------------------------------------------------------------*/
      case WM_COMMAND:
         switch (SHORT1FROMMP(mp1)) {
            case IDD_MARK_L_PUSHBUTTON:
            case IDD_MARK_C_PUSHBUTTON:
            case IDD_MARK_R_PUSHBUTTON:
            case IDD_COPY_PUSHBUTTON:
            case IDD_DELETE_PUSHBUTTON:
            case IDD_OVERLAY_PUSHBUTTON:
            case IDD_MOVE_PUSHBUTTON:
            case IDD_UPPER_PUSHBUTTON:
            case IDD_LOWER_PUSHBUTTON:
            case IDD_GOTO_PUSHBUTTON:
            case IDD_SEARCH_PUSHBUTTON:
            case IDD_FIND_PUSHBUTTON:
            case IDD_QMARK_PUSHBUTTON:
            case IDD_FMARK_PUSHBUTTON:
            {
               USHORT  usIndex;
               CHAR    szQuery[50];
               HWND    hwndCurrent;

               /* determine which button was pressed. NOTE - for this to */
               /* work all buttons must have sequential numbers with     */
               /* IDD_MARK_L_PUSHBUTTON as the first.                    */
               usIndex = SHORT1FROMMP(mp1) - IDD_MARK_L_PUSHBUTTON;

               /* process command */
               LpexCall(szCmds[usIndex], szParm[usIndex]);

               /* see which window is currently active */
               LpexQuery("WINDOWID",szQuery);
               hwndCurrent = (HWND)atol(szQuery+9);

               /* set focus back to LPEX */
               WinSetFocus(HWND_DESKTOP, WinWindowFromID(hwndCurrent,
                                                         FID_CLIENT));
            }
            break;

            /*--------------------------------------------------------*/
            /*  default action, let PM handle it                      */
            /*--------------------------------------------------------*/
            default:
               mr = WinDefDlgProc(hwndDlg, msg, mp1, mp2);
               break;
         } /* end switch */

         mr = (MRESULT) FALSE;
         break;

      default:
         mr = WinDefDlgProc(hwndDlg, msg, mp1, mp2);
         break;
   } /* end switch */

   return (mr);
}                                                      /* MarkDlgProc */
